# -*- coding: utf-8 -*-
import scrapy
import dateparser
from .base import BaseSpider
from ..items import ExaItem


class AitopSpider(BaseSpider):
    name = "aitop"
    allowed_domains = ["aitopics.org"]

    def start_requests(self):
        for i in self.companies(self.name):
            try:
                yield scrapy.Request(i.url, callback=self.parse, meta={'company': i, 'post_id': 0})
            except:
                pass

    def parse(self, response):

        company = response.meta['company']
        name = company.name
        url = 'https://aitopics.org'

        def tags(res):
            concept_tags = res.xpath(".//div[contains(@class, 'facet_concept-tagsraw')]//div//div//ul//li")
            for i in concept_tags:
                if i.xpath(".//@title").extract_first().lower() == name.lower():
                    href = i.xpath(".//a/@href").extract_first()
                    return url + href
            return None

        url_for_parse_tag = tags(response)
        if url_for_parse_tag:
            return scrapy.Request(url_for_parse_tag, callback=self.parse_by_tag, meta=response.meta)
        else:
            return scrapy.Request(response.url, callback=self.parse_by_title_description, dont_filter=True, meta=response.meta)

    def parse_by_title_description(self, response):
        company = response.meta['company']
        name = company.name

        def is_company_in_item(item):
            if name.lower() in item['title'].lower():
                return True
            elif name.lower() in item['description'].lower():
                return True
            else:
                return False

        items, is_duplicate = self.build_items(response)
        for i in items:
            if is_company_in_item(i):
                yield i

        next_url = self.next_url(response)
        if self.can_follow(next_url, is_duplicate):
            yield scrapy.Request(next_url, callback=self.parse_by_title_description, meta=response.meta)
        else:
            print("DUPLICATE NEWS")

    def parse_by_tag(self, response):
        try:
            items, is_duplicate = self.build_items(response)
            for i in items:
                yield i

            next_url = self.next_url(response)
            if self.can_follow(next_url, is_duplicate):
                yield scrapy.Request(next_url, callback=self.parse_by_tag, meta=response.meta)
            else:
                print("DUPLICATE NEWS")

        except:
            pass

    def next_url(self, response):
        has_next = response.xpath(".//ul[@class='pagination']//li/a/@href").extract()[-2]
        if has_next and has_next != '#':
            return 'https://aitopics.org/search' + has_next
        else:
            return None

    def build_items(self, response):
        rows = response.xpath(".//div[contains(@class, 'summaries')]")
        is_duplicate = False
        items = list()
        for i in rows:
            item = ExaItem()
            item['date'] = dateparser.parse(i.xpath("./div[@class='row']//time/@datetime").extract_first()).replace(
                tzinfo=None)
            item['title'] = ''.join(
                i.xpath("./div[@class='row']//div[contains(@class, 'col-xs-12')]//h3/a//text()").extract())
            item['description'] = ''.join(
                i.xpath("./div[@class='row']//div[@class='summary-content']/p//text()").extract())
            item['url'] = i.xpath(".//div[contains(@class, 'col-xs-12')]/h3/a/@href").extract_first()
            item.update(self.get_common_items(response.meta['company']))
            item['post_id'] = response.meta['post_id']
            item['tags'] = i.xpath(
                ".//div[@class='row hidden-xs']//div[@title='Concept Tags']//a[@class='filter btn btn-link']/text()").extract()

            if self.pipeline.check_url(item['url']):
                is_duplicate = True
                break
            items.append(item)
        return items, is_duplicate
